#5#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import faulthandler
faulthandler.enable()
import time
import os
os.environ['TF_CPP_MIN_LOG_LEVEL']='2'
import tensorflow as tf
import numpy as np
import tifffile as tiff
import tensorflow.keras.backend as K
from tensorflow.keras.layers import Lambda
from tensorflow.python.ops import gen_image_ops
import argparse
#import tflearn
import data_module as inputf
import random
import math
from scipy.ndimage import zoom
from scipy.ndimage import rotate
from scipy.ndimage import gaussian_filter1d
from blocks import setup_RLN
from loss import total_loss
import itertools
import tqdm



def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('--mode',  type=str, default='TS',  help="set the mode")
    parser.add_argument('--batchsizes',  type=int, default=1,  help="set the batchsize number")
    parser.add_argument('--gpu_id', type=str, default='0',   help="id(s) for CUDA_VISIBLE_DEVICES")
    parser.add_argument('--gpu_memory_limit', type=float, default=0.4, help="limit GPU memory to this fraction (0...1)")
    parser.add_argument('--train_iter', type=int, default=10000, help="set the training iteration number")
    parser.add_argument('--crop_size', type=int, default=300, help="set the large data crop size")
    parser.add_argument('--data_gen', type=bool, default=False, help="crop the train data or not")

    parser.add_argument('--main_dir', type=str, default=None, help="path to folder with input images")
    parser.add_argument('--normal_pmin', type=float, default=0.2, help="'pmin' for PercentileNormalizer")
    parser.add_argument('--normal_pmax', type=float, default=99.8, help="'pmax' for PercentileNormalizer")

    return parser, parser.parse_args()


class RLNet:
    def __init__(self):
        self.input_image = {}
        self.ground_truth = {}
        self.shape = {}
        self.cast_image = None
        self.cast_ground_truth = None
        self.cast_ground_truth1 = None
        self.loss= None
        self.prediction1, self.prediction2,self.prediction3= [None] * 3
        self.sub_diff,self.mse_square,self.mse=[None] * 3
        self.learning_rate=[None]
        self.global_step=[None]
        self.train_step = [None]
        self.loss1,self.loss2,self.loss = [None] * 3

    def set_up_rlnet(self, batch_size,fp_ch=4,bp_ch=8,merge_ch=8):
        with tf.name_scope('input'):
            output_data_channel=1
            self.shape=tf.placeholder(dtype=tf.int32)
            self.input_image = tf.placeholder(dtype=tf.float32)
            self.ground_truth = tf.placeholder(dtype=tf.float32)
            self.cast_image = tf.reshape(
                tensor=self.input_image,
                shape=[batch_size, self.shape[0], self.shape[1], self.shape[2], 1]
            )

            self.cast_ground_truth = tf.reshape(
                    tensor=self.ground_truth,
                    shape=[batch_size, self.shape[0],self.shape[1],self.shape[2],output_data_channel]
                    )

            self.cast_ground_truth1 = self.cast_ground_truth * 0.8 + 0.2 * self.cast_image

            inputs=self.cast_image

            self.prediction1, self.prediction2,self.prediction3=setup_RLN(inputs,fp_ch,bp_ch,merge_ch,batch_size,self.shape)

            self.loss1,self.loss2,self.loss=total_loss(self.cast_ground_truth1,self.prediction1,self.cast_ground_truth,self.prediction3)

            with tf.name_scope('step'):
                self.global_step = tf.Variable(0, trainable=False)
                self.learning_rate = tf.train.exponential_decay(0.025, self.global_step, 200, 0.95, staircase=False)
            with tf.control_dependencies(tf.get_collection(tf.GraphKeys.UPDATE_OPS)):
                self.train_step = tf.train.AdamOptimizer(learning_rate=self.learning_rate).minimize(self.loss,global_step=self.global_step)


    def train(self,batch_size,train_iter,train_path,normal_pmin,normal_pmax):
        if "/" in train_path:
            train_path = os.sep.join(train_path.split("/"))

        if "//" in train_path:
            train_path = os.sep.join(train_path.split("//"))

        inputf.indexi = 0
        inputf.filenames=[]
        input_path = os.path.join(train_path, 'train','input')
        inputf.filenames=os.listdir(input_path)
        inputf.filenames.sort()
        epoch = len(inputf.filenames)*10

        train_dir=os.path.join(train_path,'train','model')
        if not os.path.exists(train_dir):
            os.makedirs(train_dir)

        train_out=os.path.join(train_path,'train','output')
        if not os.path.exists(train_out):
            os.makedirs(train_out)

        checkpoint_path=os.path.join(train_dir,'model.ckpt')
        all_parameters_saver = tf.train.Saver() #save
        with tf.Session(config=config) as sess:
            sess.run(tf.global_variables_initializer())
            sess.run(tf.local_variables_initializer())
            sum_los = 0.0
            dtime=0.0
            time_start=time.time()
            for k in range(train_iter):
                t1=time.time()
                train_images,train_GT,label2out,shape=inputf.get_data_train(train_path,'train',batch_size,normal_pmin,normal_pmax)
                t2=time.time()
                dtime=dtime+t2-t1
                loss1,loss2,loss,trainer= sess.run([self.loss1,self.loss2,self.loss,self.train_step],feed_dict={self.shape:shape,self.input_image:train_images, self.ground_truth: train_GT})
#mse,ssim,lo,mse2,trainer
                sum_los += loss
                if k % 101 == 0:
                    time_end=time.time()
                    used_time=time_end-time_start
                    print('num %d, loss1: %.6f, loss2: %.6f, loss: %.6f, runtime:%.6f ' % (k, loss1, loss2, loss,used_time))
                if (k+1)%epoch==0:#13
                    print('sum_lo: %.6f' %(sum_los))
                    sum_los = 0.0
                if (k+1)%10000==0:
                    image= sess.run([self.prediction3],
                                  feed_dict={self.shape:shape,self.input_image:train_images, self.ground_truth: train_GT})
                    image1=np.array(image)
                    image1=image1.astype(np.float16)
                    print(label2out)
                    reshape_image=np.squeeze(image1)
                    if batch_size == 1:
                        single=reshape_image
                        filenames_out=os.path.join(val_out,'rl_'+label2out[0])
                        tiff.imsave(filenames_out,single)
                    else:    
                        for v in range(batch_size):
                            single=reshape_image[v]
                            filenames_out=os.path.join(val_out,'rl_'+label2out[v])
                            tiff.imsave(filenames_out,single)
                if (k+1)%epoch == 0:
                    all_parameters_saver.save(sess=sess, save_path=checkpoint_path)
            print("Done training")
        sess.close()

    def valid(self,val_path,normal_pmin=0.05,normal_pmax=99.5,batch_size=1):
        if "/" in val_path:
            val_path = os.sep.join(val_path.split("/"))

        if "//" in val_path:
            val_path = os.sep.join(val_path.split("//"))

        inputf.filenames=[]
        inputf.indexi=0
        input_path = os.path.join(val_path, 'test','input')
        inputf.filenames=os.listdir(input_path)
        inputf.filenames.sort()
        val_iter = len(inputf.filenames)

        model_dir=os.path.join(val_path,'train','model')
        checkpoint_path=os.path.join(model_dir,'model.ckpt')

        val_out=os.path.join(val_path,'test','output')
        if not os.path.exists(val_out):
            os.makedirs(val_out)

        all_parameters_saver = tf.train.Saver()
        with tf.Session() as sess:  # 开始一个会话
            sess.run(tf.global_variables_initializer())
            sess.run(tf.local_variables_initializer())
            all_parameters_saver.restore(sess=sess, save_path=checkpoint_path)
            sum_los = 0.0
            for m in range(val_iter):
                time_start=time.time()
                test_images,test_GT,label2out,shape=inputf.get_data_valid(val_path,'test',batch_size,normal_pmin,normal_pmax)
                image, los = sess.run([self.prediction3, self.loss],feed_dict={self.shape:shape,self.input_image:test_images, self.ground_truth: test_GT})
                sum_los += los
                image1=np.array(image)
                image1=image1.astype(np.float16)
                print(image1.shape)
                reshape_image=np.squeeze(image1)
                if batch_size == 1:
                    single=reshape_image
                    filenames_out=os.path.join(val_out,'rl_'+label2out[0])
                    tiff.imsave(filenames_out,single)
                else:    
                    for v in range(batch_size):
                        single=reshape_image[v]
                        filenames_out=os.path.join(val_out,'rl_'+label2out[v])
                        tiff.imsave(filenames_out,single)
                if m % 1 == 0:
                    time_end=time.time()
                    print('num %d, loss: %.6f, runtime:%.6f ' % (m, los,time_end-time_start))
        print('Done validating')


    def test(self,test_path,normal_pmin=0.05,normal_pmax=99.5,batch_size=1):
        if "/" in test_path:
            test_path = os.sep.join(test_path.split("/"))

        if "//" in test_path:
            test_path = os.sep.join(test_path.split("//"))

        inputf.indexi = 0
        inputf.filenames=[]
        input_path = os.path.join(test_path, 'test','input')
        inputf.filenames=os.listdir(input_path)
        inputf.filenames.sort()
        test_iter = len(inputf.filenames)

        model_dir=os.path.join(test_path,'train','model')
        checkpoint_path=os.path.join(model_dir,'model.ckpt')

        test_out=os.path.join(test_path,'test','output')
        if not os.path.exists(test_out):
            os.makedirs(test_out)

        all_parameters_saver = tf.train.Saver()
        with tf.Session() as sess:  # 开始一个会话
            sess.run(tf.global_variables_initializer())
            sess.run(tf.local_variables_initializer())
            all_parameters_saver.restore(sess=sess, save_path=checkpoint_path)
            for m in range(test_iter):
                time_start=time.time()
                max_v,test_images,label2out,shape=inputf.get_data_test(test_path,'test',batch_size,normal_pmin,normal_pmax)
                image = sess.run([self.prediction3],feed_dict={self.shape:shape,self.input_image:test_images})
                image1=np.array(image)
                image1=image1.astype(np.float16)
                print(image1.shape)
                reshape_image=np.squeeze(image1)
                if batch_size == 1:
                    single=reshape_image
                    filenames_out=os.path.join(test_out,'rl_'+label2out[0])
                    tiff.imsave(filenames_out,single)
                else:    
                    for v in range(batch_size):
                        single=reshape_image[v]
                        filenames_out=os.path.join(test_out,'rl_'+label2out[v])
                        tiff.imsave(filenames_out,single)
                if m % 1 == 0:
                    time_end=time.time()
                    print('num %d, runtime:%.6f ' % (m, time_end-time_start))
        print('Done testing')


    def test_large(self,test_path,normal_pmin=0.05,normal_pmax=99.5,crop_data_size=300,batch_size=1):
        image_dim = 3
        num_output_channels=1

        inputf.indexi = 0
        inputf.filenames=[]
        input_path = os.path.join(test_path, 'test','input')
        inputf.filenames=os.listdir(input_path)
        inputf.filenames.sort()
        test_iter = len(inputf.filenames)
        model_dir = os.path.join(test_path, 'train', 'model')
        checkpoint_path = os.path.join(model_dir, 'model.ckpt')

        test_out = os.path.join(test_path, 'test', 'output')
        if not os.path.exists(test_out):
            os.makedirs(test_out)

        variables=tf.contrib.framework.get_variables_to_restore()
        variables_to_restore1=[v for v in variables if (v.name.split('/')[0]!='step')]
        all_parameters_saver = tf.train.Saver(variables_to_restore1)
        with tf.Session() as sess:  # 开始一个会话
            time_start_init = time.time()
            sess.run(tf.global_variables_initializer())
            sess.run(tf.local_variables_initializer())
            all_parameters_saver.restore(sess=sess, save_path=checkpoint_path)
            time_read_all = 0
            time_crop_all = 0
            time_process_all = 0
            time_save_all = 0

            for nn in range(0, test_iter):
                time_start = time.time()
                input_file = os.path.join(input_path,inputf.filenames[nn])
                print(input_file)
                label_out = inputf.filenames[nn]

                time_read_s = time.time()
                input_tif = tiff.imread(input_file)  # the input large data_size
                time_read_e = time.time()
                time_read_all = time_read_all + time_read_e - time_read_s

                output_image_shape = input_tif.shape
                input_image_shape = input_tif.shape

                time_crop_s = time.time()
                if input_image_shape[0] < 40:
                    d = 0
                else:
                    d = 10
                overlap_shape = (d, 24, 24)
                if input_image_shape[1] % 2 != 0:
                    overlap_shape = (d, 23, 24)
                if input_image_shape[2] % 2 != 0:
                    overlap_shape = (d, 32, 24)
                # set the cropped size
                crop_d = min(output_image_shape[0], 2000)
                crop_w = min(math.floor(math.sqrt(crop_data_size / 4 * 1024 * 1024 / crop_d)), 1998)
                crop_h = min(math.floor(math.sqrt(crop_data_size / 4 * 1024 * 1024 / crop_d)), 1998)
                if crop_w % 2 != 0:
                    different = 2 - crop_w % 2
                    crop_w = crop_w - different
                if crop_h % 2 != 0:
                    different = 2 - crop_h % 2
                    crop_h = crop_h - different
                model_input_image_shape = (crop_d, crop_h, crop_w)
                print(model_input_image_shape)
                step_shape = tuple(m - o for m, o in zip(model_input_image_shape, overlap_shape))

                block_weight = np.ones(
                    [m - 2 * o for m, o
                     in zip(model_input_image_shape, overlap_shape)], dtype=np.float32)
                block_weight = np.pad(
                    block_weight,
                    [(o + 1, o + 1) for o in overlap_shape],
                    'linear_ramp')[(slice(1, -1),) * image_dim]


                applied = np.zeros(
                    (*output_image_shape, num_output_channels), dtype=np.float32)
                sum_weight = np.zeros(output_image_shape, dtype=np.float32)
                num_steps = tuple(
                    i // s + (((i // s) * s + o) < i)
                    for i, s, o in zip(input_image_shape, step_shape, overlap_shape))

                blocks = list(itertools.product(
                    *[np.arange(n) * s for n, s in zip(num_steps, step_shape)]))

                print(blocks)

                time_crop_e = time.time()
                time_crop_all = time_crop_all + time_crop_e - time_crop_s

                time_process_s = time.time()

                for chunk_index in tqdm.trange(
                        0, len(blocks),batch_size, disable=False,
                        dynamic_ncols=True, ascii=tqdm.utils.IS_WIN):
                    rois = []
                    maxv = []
                    minv = []
                    for batch_index, tl in enumerate(blocks[chunk_index:chunk_index + batch_size]):
                        # tl 左上角坐标， br 右下角坐标
                        br = [min(t + m, i) for t, m, i in zip(tl, model_input_image_shape, input_image_shape)]
                        # r1是用于预测的图像区域
                        r1, r2 = zip(*[(slice(s, e), slice(0, e - s)) for s, e in zip(tl, br)])
                        # print(r2)

                        m = input_tif[r1]
                        block_weight1 = block_weight
                        # print(m.shape)
                        if model_input_image_shape != m.shape:
                            # reshape the weight block
                            block_weight1 = block_weight[:m.shape[0], :m.shape[1], :m.shape[2]]

                            # expand the data
                            # pad_width = [(0, b - s) for b, s
                            #             in zip(model_input_image_shape, m.shape)]
                            # print(pad_width)
                            # m = np.pad(m, pad_width, 'reflect')

                        shape_in = m.shape
                        # print(shape_in)
                        min_v, max_v, normal_input_tif = inputf.normalize(m, normal_pmin, normal_pmax)
                        batch = normal_input_tif
                        rois.append((r1, r2))
                        maxv.append(max_v)
                        minv.append(min_v)

                    # time_start = time.time()
                    image = sess.run(self.prediction3,
                                     feed_dict={self.shape: shape_in, self.input_image: batch})
                    #                    if maxv[0]>2*minv[0]:
                    #                        image = image*maxv[0]-minv[0]
                    #                    else:
                    image = image * maxv[0]
                    # image[image<0]=0
                    # print('num %d, runtime:%.6f ' % (nn,time_end-time_start))
                    for batch_index in range(len(rois)):
                        for channel in range(num_output_channels):
                            image[batch_index, ..., channel] *= block_weight1

                        r1, r2 = [roi for roi in rois[batch_index]]
                        # print(applied[r1].shape,image[batch_index][r2].shape)

                        applied[r1] += image[batch_index][r2]
                        sum_weight[r1] += block_weight[r2]

                time_process_e = time.time()
                time_process_all = time_process_all + time_process_e - time_process_s
                time_save_s = time.time()
                for channel in range(num_output_channels):
                    applied[..., channel] /= sum_weight

                if applied.shape[-1] == 1:
                    applied = applied[..., 0]

                image1 = np.array(applied)
                image1 = np.squeeze(image1)
                single = image1.astype(np.float16)
                print(single.shape)
                filenames_out =os.path.join(test_out , 'RLN_' + label_out)
                tiff.imsave(filenames_out, single)
                time_save_e = time.time()
                time_save_all = time_save_all + time_save_e - time_save_s
                time_end = time.time()
                print('num %d, runtime_all:%.6f ' % (nn, time_end - time_start))
            print('total_time: %.6f' % (time.time() - time_start_init))
            print('time_read_all: %.6f,time_crop_all: %.6f,time_process_all: %.6f,time_save_all: %.6f' % (
            time_read_all, time_crop_all, time_process_all, time_save_all))
        print('Done testing')


if __name__=="__main__":
    parser, args = parse_args()

    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = args.gpu_id

    config = tf.ConfigProto()
    config.gpu_options.per_process_gpu_memory_fraction = args.gpu_memory_limit

    path_main=args.main_dir
    if args.data_gen:
        path_in=os.path.join(args.main_dir, 'train')
        path_out=os.path.join(args.main_dir, 'train1')
        inputf.data_gen(64, 64, 64, 10, path_in, path_out)
        os.rename(path_in,os.path.join(args.main_dir, 'train_old'))
        os.rename(path_out,os.path.join(args.main_dir, 'train'))

    mode=args.mode
    net = RLNet()
    if mode == 'TR':
        net.set_up_rlnet(args.batchsizes)
        net.train(args.batchsizes,args.train_iter,args.main_dir,args.normal_pmin,args.normal_pmax)
    if mode == 'VL':
        net.set_up_rlnet(args.batchsizes)
        net.valid(args.main_dir,args.normal_pmin,args.normal_pmax)
    if mode == 'TS':
        net.set_up_rlnet(args.batchsizes)
        net.test(args.main_dir,args.normal_pmin,args.normal_pmax)
    if mode == 'TSS':
        net.set_up_rlnet(args.batchsizes)
        net.test_large(args.main_dir,args.normal_pmin,args.normal_pmax,args.crop_size)








#5#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import tensorflow as tf
import numpy as np
import tensorflow.keras.backend as K
from tensorflow.keras.layers import Lambda
from internals import init_w, gaussian_ker, fft3d, ifft3d


@tf.custom_gradient
def tensor_div(a,x1):
    m=tf.div_no_nan(a, x1+0.0001)

    def grad(dy):
        return dy,-tf.square(m)*dy
    return m, grad


@tf.custom_gradient
def tensor_mul(a,x):
    m=tf.multiply(a, x)

    def grad(dy):
        return dy,dy*(a)
    return m, grad


def chan_ave(x):
    xo=tf.reduce_mean(x,axis=-1,keep_dims=True)
    return xo

def _name(s):
    return s

def activationF(x, name='leaky_relu'):
    return tf.nn.softplus(x)


def batch_norm(x):
    return tf.contrib.layers.layer_norm(x)


def avg_pool(x):
    output=tf.nn.avg_pool3d(x,[1,2,2,2,1],strides=[1,2,2,2,1],padding='VALID',data_format='NDHWC')
    return output


def FP_add_part(x,y):
    bs, _, _, _, ch = x.get_shape().as_list()
    y=activationF(tf.tile(input=y,multiples=[1,1,1,1,ch]))
    output=x+y
    return output


def FP_H1(x,ch):
    with tf.name_scope('FP_H1'):
        inputs=avg_pool(x)
        kernel1=gaussian_ker(shape=[3,3,3,1,ch], name='e_1')
        kernel2=gaussian_ker(shape=[3,3,3,ch,ch], name='e_2')
        kernel3=gaussian_ker(shape=[3,3,3,2*ch,ch], name='e_3')

        layer=tf.nn.conv3d(input=inputs, filter=kernel1,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_1')
        layer=batch_norm(layer)
        layer=activationF(layer)

        temp1=layer

        layer=tf.nn.conv3d(input=layer, filter=kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.concat([layer,temp1],-1)

        layer=tf.nn.conv3d(input=layer, filter=kernel3,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=FP_add_part(layer,inputs)

        return layer


def DV_H1(x,y):
    with tf.name_scope('BP_H1'):
        devidend=avg_pool(x)
        devisor=chan_ave(y)
        output=tensor_div(devidend, devisor)
        output=batch_norm(output)
        return output


def BP_H1(x,ch,shapes):
    with tf.name_scope('BP_H1'):
        kernel1=init_w(shape=[3,3,3,1,ch], name='e_1')
        kernel2=init_w(shape=[3,3,3,ch,ch], name='e_2')
        kernel3=init_w(shape=[3,3,3,2*ch,ch], name='e_2')
        up_kernel1 = init_w(shape=[2, 2, 2, ch//2, ch], name='up_1')
        up_kernel2 = init_w(shape=[3, 3, 3, ch //2, ch//2], name='up_2')

        layer=tf.nn.conv3d(input=x, filter=kernel1,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_1')
        layer=batch_norm(layer)
        layer=activationF(layer)

        temp1=layer

        layer=tf.nn.conv3d(input=layer, filter=kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.concat([layer,temp1],-1)

        layer=tf.nn.conv3d(input=layer, filter=kernel3,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_3')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.nn.conv3d_transpose(value=layer, filter=up_kernel1,output_shape=shapes,strides=[1, 2, 2, 2, 1], padding='VALID', name='up_1')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.nn.conv3d(input=layer, filter=up_kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='up_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        output=chan_ave(layer)

        return output


def update_H1(x,y):
    with tf.name_scope('update_H1'):
        output=tf.multiply(x,y)
        return output


def FP_H2(x,ch):
    with tf.name_scope('FP_H2'):
        kernel1=gaussian_ker(shape=[3,3,3,1,ch], name='e_1')
        kernel2=gaussian_ker(shape=[3,3,3,ch,ch], name='e_2')

        inputs=x
        layer=tf.nn.conv3d(input=inputs, filter=kernel1,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_1')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.nn.conv3d(input=layer, filter=kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=FP_add_part(layer,inputs)

        return layer


def DV_H2(x,y):
    with tf.name_scope('BP_H2'):
        devidend=x
        devisor=chan_ave(y)
        output=tensor_div(devidend, devisor)
        output=batch_norm(output)
        return output


def BP_H2(x,ch):
    with tf.name_scope('BP_H2'):
        kernel1=init_w(shape=[3,3,3,1,ch], name='e_1')
        kernel2=init_w(shape=[3,3,3,ch,ch], name='e_2')

        layer=tf.nn.conv3d(input=x, filter=kernel1,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_1')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.nn.conv3d(input=layer, filter=kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)
        layer=layer+tf.ones_like(layer)

        output=chan_ave(layer)

        return output


def update_H2(x,y):
    with tf.name_scope('update_H2'):
        x=batch_norm(x)
        x=activationF(x)
        layer=tf.multiply(x,y)
        layer=batch_norm(layer)
        output=activationF(layer)
        return output


def merge_H3(e1,e2,ch):
    with tf.name_scope('BP_H2'):
        e1=batch_norm(e1)
        e1=activationF(e1)

        kernel1=init_w(shape=[3,3,3,1,ch], name='e_1')
        kernel2=init_w(shape=[3,3,3,ch+2,ch], name='e_2')
        kernel3=init_w(shape=[3,3,3,2*ch,ch], name='e_3')

        layer=tf.nn.conv3d(input=e2, filter=kernel1,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_1')
        layer=batch_norm(layer)
        layer=activationF(layer)
        temp1=layer

        layer=tf.concat([layer,e1,e2],-1)

        layer=tf.nn.conv3d(input=layer, filter=kernel2,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_2')
        layer=batch_norm(layer)
        layer=activationF(layer)

        layer=tf.concat([layer,temp1],-1)

        layer=tf.nn.conv3d(input=layer, filter=kernel3,strides=[1, 1, 1, 1, 1], padding='SAME', name='conv_3')
        layer=batch_norm(layer)
        layer=activationF(layer)

        output=chan_ave(layer)

        return output


def global_avg_pool(x):
    b,_,_,_,c=x.shape
    return tf.reshape(tf.reduce_mean(x,axis=[1,2,3]),(b,1,c))


def mul_attention1(inputs,index,inner_units_ratio=0.5): #channel_attention
    with tf.variable_scope("CAB%s"%(index)):
        inputs_shape = map(lambda x: x.value, inputs.shape)
        b,d,h,w,c=inputs_shape
        GAP=global_avg_pool(inputs)
        fc_1=tf.layers.dense(inputs=GAP,units=c*inner_units_ratio,name="fc_1",activation=None)
        fc_1=tf.nn.relu(fc_1)
        fc_2=tf.layers.dense(inputs=fc_1,units=c,name="fc_2",activation=None)
        fc_2=tf.nn.sigmoid(fc_2)
        channel_attention=tf.reshape(fc_2, shape=[b,1, 1, 1,c])
        out1= tf.multiply(inputs, channel_attention)
        return out1


def mul_attention_FCA(inputs, index, inner_units_ratio=8.0):
    with tf.variable_scope("CAB%s" % (index)):
        b, _, _, _, c = inputs.get_shape().as_list()
        d = tf.shape(inputs)[1]
        h = tf.shape(inputs)[2]
        w = tf.shape(inputs)[3]
        absfft1 = Lambda(fft3d, arguments={'gamma': 1.0})(inputs)

        mean_map = tf.reduce_mean(absfft1, axis=-1, keep_dims=True)
        max_map = tf.reduce_max(absfft1, axis=-1, keep_dims=True)
        merge_map = tf.concat([mean_map, max_map], -1)
        kernels1 = init_w(shape=[3, 3, 3, 2, 2], name=_name("inside_kernel1%s" % (index)))
        kernels2 = init_w(shape=[3, 3, 3, 2, 2], name=_name("inside_kernel2%s" % (index)))
        kernels3 = init_w(shape=[3, 3, 3, 2, 2], name=_name("inside_kernel3%s" % (index)))
        kernels4 = init_w(shape=[3, 3, 3, 6, c], name=_name("inside_kernel4%s" % (index)))
        inside_conv1 = tf.nn.conv3d(
            input=merge_map, filter=kernels1,
            strides=[1, 1, 1, 1, 1], padding='SAME', name=_name("conv_1%s" % (index)))
        inside_conv2 = tf.nn.conv3d(
            input=inside_conv1, filter=kernels2,
            strides=[1, 1, 1, 1, 1], padding='SAME', name=_name("conv_2%s" % (index)))
        inside_conv3 = tf.nn.conv3d(
            input=inside_conv2, filter=kernels3,
            strides=[1, 1, 1, 1, 1], padding='SAME', name=_name("conv_3%s" % (index)))
        merge_map_all = tf.concat([inside_conv1, inside_conv2, inside_conv3], -1)
        inside_conv = tf.nn.conv3d(
            input=merge_map_all, filter=kernels4,
            strides=[1, 1, 1, 1, 1], padding='SAME', name=_name("conv_%s" % (index)))
        inside_ac = tf.nn.sigmoid(batch_norm(x=inside_conv))  # tf.nn.sigmoid(inside_conv)
        out1 = tf.multiply(absfft1, inside_ac)

        out2 = Lambda(ifft3d, arguments={'gamma': 1.0})(out1)
        out2 = out2 / (tf.reduce_max(out2) + 0.001)

        out3 = mul_attention1(out2, index, 2)
        inputs = mul_attention1(inputs, index + 10, 2)
        out4 = tf.concat([inputs, out3], -1)
        kernels5 = init_w(shape=[3, 3, 3, 2 * c, c], name=_name("inside_kernel5%s" % (index)))
        output = tf.nn.conv3d(
            input=out4, filter=kernels5,
            strides=[1, 1, 1, 1, 1], padding='SAME', name=_name("conv_1%s" % (index)))
        output = activationF(batch_norm(x=output))

        return output


def setup_RLN(x,fp_ch,bp_ch,merge_ch,batchsizes,input_shape):
    inputs=x
    shapes = [batchsizes, input_shape[0], input_shape[1], input_shape[2], bp_ch//2]
    fp1=FP_H1(inputs,fp_ch)
    dv1=DV_H1(inputs, fp1)
    bp1=BP_H1(dv1, bp_ch, shapes)
    update1=update_H1(inputs,bp1)

    fp2 = FP_H2(inputs, fp_ch)
    dv2 = DV_H2(inputs, fp2)
    bp2 = BP_H2(dv2, bp_ch,)
    update2 = update_H2(update1, bp2)

    mergeh3=merge_H3(update1,update2,merge_ch)

    return update1, update2, mergeh3



#5#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import tensorflow as tf
import numpy as np
import tensorflow.keras.backend as K


def init_w(shape, name, stddev=1.0):  # 0.5和1都可以
    with tf.name_scope('init_w'):
        w = tf.Variable(initial_value=tf.truncated_normal(shape=shape, mean=0.0, stddev=stddev, dtype=tf.float32),
                        name=name)
        return w


def tf_fspecial_gauss(size, sigma1):
    # define 3D gaussian function block
    x_data, y_data, z_data = np.mgrid[-size[0]//2 + 1:size[0]//2 + 1, -size[1]//2 + 1:size[1]//2 + 1,-size[2]//2 + 1:size[2]//2 + 1]
    x_data = np.expand_dims(x_data, axis=-1)
    x_data = np.expand_dims(x_data, axis=-1)

    y_data = np.expand_dims(y_data, axis=-1)
    y_data = np.expand_dims(y_data, axis=-1)

    z_data = np.expand_dims(z_data, axis=-1)
    z_data = np.expand_dims(z_data, axis=-1)

    x = tf.constant(x_data, dtype=tf.float32)
    y = tf.constant(y_data, dtype=tf.float32)
    z = tf.constant(z_data, dtype=tf.float32)

    g = tf.exp(-((x**2 + y**2+z**2)/(2.0*sigma1**2)))
    g=g/tf.reduce_max(g)
    return g


def gaussian_ker(shape,name,stddev=1):#1
    c=shape[4]
    init=tf_fspecial_gauss(shape,stddev*0.5)
    for i in range(2,c+1):
        kernel=tf_fspecial_gauss(shape,stddev*0.5*i)
        init = tf.concat([init,kernel], -1)
    rad=tf.random_uniform([1,1,1,shape[3],1],minval=0.7,maxval=1,dtype=tf.float32)
    init=tf.tile(init,[1,1,1,shape[3],1])*rad
    w =tf.Variable(initial_value=init, name=name)
    return w


def copy_and_crop_and_merge(result_from_downsampling, result_from_upsampling):
    return tf.concat(values=[result_from_downsampling, result_from_upsampling], axis=-1)


## FFT function
def fft2d(input, gamma=0.1):
    temp = K.permute_dimensions(input, (0, 3, 1, 2))
    fft = tf.signal.fft2d(tf.complex(temp, tf.zeros_like(temp)))
    absfft = tf.pow(tf.abs(fft)+1e-8, gamma)
    output = K.permute_dimensions(absfft, (0, 2, 3, 1))
    return output


def fft3d(input, gamma=0.1):
    input = apodize3d(input, napodize=5)
    temp = K.permute_dimensions(input, (0, 4, 1, 2, 3))
    fft = tf.signal.fft3d(tf.complex(temp, tf.zeros_like(temp)))
    absfft = tf.pow(tf.abs(fft) + 1e-8, gamma)
    output = K.permute_dimensions(absfft, (0, 2, 3, 4, 1))
    return output


def ifft3d(input, gamma=0.1):
    input = apodize3d(input, napodize=5)
    temp = K.permute_dimensions(input, (0, 4, 1, 2, 3))
    fft = tf.signal.ifft3d(tf.complex(temp, tf.zeros_like(temp)))
    absfft = tf.pow(tf.abs(fft) + 1e-8, gamma)
    output = K.permute_dimensions(absfft, (0, 2, 3, 4, 1))
    return output


def fftshift2d(input,size_psc=128):
    h=tf.shape(input)[1]
    w=tf.shape(input)[2]
    fs11 = input[:, -h // 2:h, -w // 2:w, :]
    fs12 = input[:, -h // 2:h, 0:w // 2, :]
    fs21 = input[:, 0:h // 2, -w // 2:w, :]
    fs22 = input[:, 0:h // 2, 0:w // 2, :]
    output = tf.concat([tf.concat([fs11, fs21], axis=1), tf.concat([fs12, fs22], axis=1)], axis=2)
    output = tf.image.resize_images(output, (size_psc, size_psc), 0)
    return output


def fftshift3d(input,size_psc=[64,64,64]):
    bs, _, _, _, ch = input.get_shape().as_list()
    h=tf.shape(input)[1]
    w=tf.shape(input)[2]
    z=tf.shape(input)[3]
    fs111 = input[:, -h // 2:h, -w // 2:w, -z // 2 + 1:z, :]
    fs121 = input[:, -h // 2:h, 0:w // 2, -z // 2 + 1:z, :]
    fs211 = input[:, 0:h // 2, -w // 2:w, -z // 2 + 1:z, :]
    fs221 = input[:, 0:h // 2, 0:w // 2, -z // 2 + 1:z, :]
    fs112 = input[:, -h // 2:h, -w // 2:w, 0:z // 2 + 1, :]
    fs122 = input[:, -h // 2:h, 0:w // 2, 0:z // 2 + 1, :]
    fs212 = input[:, 0:h // 2, -w // 2:w, 0:z // 2 + 1, :]
    fs222 = input[:, 0:h // 2, 0:w // 2, 0:z // 2 + 1, :]
    output1 = tf.concat([tf.concat([fs111, fs211], axis=1), tf.concat([fs121, fs221], axis=1)], axis=2)
    output2 = tf.concat([tf.concat([fs112, fs212], axis=1), tf.concat([fs122, fs222], axis=1)], axis=2)
    output0 = tf.concat([output1, output2], axis=3)
##original 2
    output = []
    output=tf.reshape(output0,[bs,h,w,-1] )#tf.map_fn(lambda x:tf.image.resize_images(x, (size_psc, size_psc),0),elems=output0)
    output=tf.image.resize_images(output, (size_psc[0], size_psc[1]),0)
    output=tf.reshape(output,[bs,size_psc[0],size_psc[1],z,ch] )
    output=tf.reshape(output,[bs*size_psc[0],size_psc[1],z,ch] )
    output=tf.image.resize_images(output, (size_psc[1], size_psc[2]),0)
    output=tf.reshape(output,[bs,size_psc[0],size_psc[1],size_psc[2],ch] )
##original 1
#    for iz in tf.range(0,z1):
#        output.append(tf.image.resize_images(output0[:, :, :, iz, :], (size_psc, size_psc), 0))
#    output = tf.stack(output, axis=0)
    return output


def apodize2d(img, napodize=10):
    bs, ny, nx, ch = img.get_shape().as_list()
    img_apo = img[:, napodize:ny-napodize, :, :]

    imageUp = img[:, 0:napodize, :, :]
    imageDown = img[:, ny-napodize:, :, :]
    diff = (imageDown[:, -1::-1, :, :] - imageUp) / 2
    l = np.arange(napodize)
    fact_raw = 1 - np.sin((l + 0.5) / napodize * np.pi / 2)
    fact = fact_raw[np.newaxis, :, np.newaxis, np.newaxis]
    fact = tf.convert_to_tensor(fact, dtype=tf.float32)
    fact = tf.tile(fact, [tf.shape(img)[0], 1, nx, ch])
    factor = diff * fact
    imageUp = tf.add(imageUp, factor)
    imageDown = tf.subtract(imageDown, factor[:, -1::-1, :, :])
    img_apo = tf.concat([imageUp, img_apo, imageDown], axis=1)

    imageLeft = img_apo[:, :, 0:napodize, :]
    imageRight = img_apo[:, :, nx-napodize:, :]
    img_apo = img_apo[:, :, napodize:nx-napodize, :]
    diff = (imageRight[:, :, -1::-1, :] - imageLeft) / 2
    fact = fact_raw[np.newaxis, np.newaxis, :, np.newaxis]
    fact = tf.convert_to_tensor(fact, dtype=tf.float32)
    fact = tf.tile(fact, [tf.shape(img)[0], ny, 1, ch])
    factor = diff * fact
    imageLeft = tf.add(imageLeft, factor)
    imageRight = tf.subtract(imageRight, factor[:, :, -1::-1, :])
    img_apo = tf.concat([imageLeft, img_apo, imageRight], axis=2)

    return img_apo


def apodize3d(img, napodize=5):
    bs, _, _, _, ch = img.get_shape().as_list()
    ny=tf.shape(img)[1]
    nx=tf.shape(img)[2]
    nz=tf.shape(img)[3]
    img_apo = img[:, napodize:ny-napodize, :, :, :]
    imageUp = img[:, 0:napodize, :, :, :]
    imageDown = img[:, ny-napodize:, :, :, :]
    diff = (imageDown[:, -1::-1, :, :, :] - imageUp) / 2
    l = np.arange(napodize)
    fact_raw = 1 - np.sin((l + 0.5) / napodize * np.pi / 2)
    fact = fact_raw[np.newaxis, :, np.newaxis, np.newaxis, np.newaxis]
    fact = tf.convert_to_tensor(fact, dtype=tf.float32)
    fact = tf.tile(fact, [tf.shape(img)[0], 1, nx, nz, ch])
    factor = diff * fact
    imageUp = tf.add(imageUp, factor)
    imageDown = tf.subtract(imageDown, factor[:, -1::-1, :, :, :])
    img_apo = tf.concat([imageUp, img_apo, imageDown], axis=1)
    imageLeft = img_apo[:, :, 0:napodize, :, :]
    imageRight = img_apo[:, :, nx-napodize:, :, :]
    img_apo = img_apo[:, :, napodize:nx-napodize, :, :]
    diff = (imageRight[:, :, -1::-1, :, :] - imageLeft) / 2
    fact = fact_raw[np.newaxis, np.newaxis, :, np.newaxis, np.newaxis]
    fact = tf.convert_to_tensor(fact, dtype=tf.float32)
    fact = tf.tile(fact, [tf.shape(img)[0], ny, 1, nz, ch])
    factor = diff * fact
    imageLeft = tf.add(imageLeft, factor)
    imageRight = tf.subtract(imageRight, factor[:, :, -1::-1, :, :])
    img_apo = tf.concat([imageLeft, img_apo, imageRight], axis=2)
    return img_apo


#5#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import tensorflow as tf
import numpy as np
import tensorflow.keras.backend as K


def get_mse(gt_label, dl_op):
    sub_diff = dl_op - gt_label
    mse_square = tf.square(sub_diff)
    mse = tf.reduce_mean(mse_square) + 0.0001
    return mse


def get_mae(gt_label,dl_op):
    sub_diff =tf.abs(dl_op-gt_label)
    mae=tf.reduce_mean(sub_diff)
    return mae


def get_SSIM(gt_label, dl_op,max_val=1):
    mean_prediction =tf.reduce_mean(dl_op)
    mean_gt =tf.reduce_mean(gt_label)
    sigma_prediction=tf.reduce_mean(tf.square(tf.subtract(dl_op,mean_prediction)))
    sigma_gt=tf.reduce_mean(tf.square(tf.subtract(gt_label,mean_gt)))
    sigma_cross=tf.reduce_mean(tf.multiply(tf.subtract(dl_op,mean_prediction),
                                                    tf.subtract(gt_label,mean_gt)))
    ssim_1=2*tf.multiply(mean_prediction,mean_gt)+1e-4*max_val*max_val
    ssim_2=2*sigma_cross+9e-4**max_val*max_val
    ssim_3=tf.square(mean_prediction)+tf.square(mean_gt)+1e-4**max_val*max_val
    ssim_4=sigma_prediction+sigma_gt+9e-4**max_val*max_val
    ssim=tf.div(tf.multiply(ssim_1,ssim_2),tf.multiply(ssim_3,ssim_4))
    return ssim


def total_loss(x1,y1,x2,y2):
    loss1=get_mse(x1, y1)
    loss2=get_mse(x2, y2)+1.0*(1-get_SSIM(x2,y2))
    loss=0.2*loss1+loss2
    return loss1,loss2,loss




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import tensorflow as tf
import numpy as np
#import matplotlib.pyplot as plt
import random
import tifffile as tiff

FLAGS = tf.app.flags.FLAGS
batch_index = 0
indexi=0
filenames = []
input_folder='input'
ground_truth_folder='ground_truth'


def get_filenames(data_dir,data_set): #获取文件名称
    global filenames
    filetype='.tif'
    for root, dirs, files in os.walk(os.path.join(data_dir,data_set,input_folder)):
        for file in files:
            if file.endswith(filetype):
                filenames.append(file)
    random.shuffle(filenames)


def normalize_mi_ma(s,mi,ma,eps=1e-20,dtype=np.float32):
    x=(s-mi)/(ma-mi+eps)
    return x


def normalize(x,pmin=0.0,pmax=99.6,axis=None,eps=1e-20,dtype=np.float32):
    mi=np.percentile(x,pmin,axis=axis,keepdims=True)
    ma=np.percentile(x,pmax,axis=axis,keepdims=True)
    mean_v=x.mean()
    while(pmin> 0.0 and ma-mi<0.05*mean_v):
        mi=np.percentile(x,0.05,axis=axis,keepdims=True)
    return mi,ma,normalize_mi_ma(x,mi,ma,eps=eps,dtype=dtype)


def random_crop(x,y,b_d=64,b_h=64,b_w=64):
    w=max(0,(x.shape[0]-b_d)//16)
    h=max(0,(x.shape[1]-b_h)//16)
    d=max(0,(x.shape[2]-b_w)//16)
    x1=random.randint(0,max(0,w-1))*16
    y1=random.randint(0,max(0,h-1))*16
    z1=random.randint(0,max(0,d-1))*16
    x2=x1+b_d #新的结尾坐标
    y2=y1+b_h
    z2=z1+b_w
    if x2>x.shape[0]:
        x1=0
        x2=x.shape[0]-1
    if y2>x.shape[1]:
        y1=0
        y2=x.shape[1]-1
    if z2>x.shape[2]:
        z1=0
        z2=x.shape[2]-1
    r_x=x[x1:x2,y1:y2,z1:z2]
    r_y=y[x1:x2,y1:y2,z1:z2]
    return r_x,r_y


def data_aug_online(x,y,b_d=64,b_h=64,b_w=64):
    mean_whole=y.mean()
    x1,y1=random_crop(x,y,b_d,b_h,b_w)
    while (y1.mean()<mean_whole*0.8):
        x1,y1=random_crop(x,y,b_d,b_h,b_w)
    return x1,y1


def data_gen(b_d,b_h,b_w,crop_time,path_in,path_out):
    if "/" in path_in:
        path_in=os.sep.join(path_in.split("/"))

    if "//" in path_in:
        path_in=os.sep.join(path_in.split("//"))

    if "/" in path_out:
        path_out=os.sep.join(path_out.split("/"))

    if "//" in path_out:
        path_out=os.sep.join(path_out.split("//"))

    path_in1=os.path.join(path_in,'input')
    path_in2 = os.path.join(path_in, 'ground_truth')

    path_out1=os.path.join(path_out,'input')
    path_out2 = os.path.join(path_out, 'ground_truth')

    if not os.path.exists(path_out1):
        os.makedirs(path_out1)

    if not os.path.exists(path_out2):
        os.makedirs(path_out2)

    datanames = os.listdir(path_in1)

    for i in datanames:
        filename_main=i
        input_path=path_in1+filename_main
        decon_path = path_in2+ filename_main

        low_tif = tiff.imread(input_path)
        decon_tif=tiff.imread(decon_path)

        for j in range(crop_time):
            low_tiff_c,decon_tif_c=data_aug_online(low_tif,decon_tif,b_d,b_h,b_w)

            low_out = path_input_out + i[0:-4]+'-'+str(j)+'.tif'
            decon_out=path_decon_out+i[0:-4]+'-'+str(j)+'.tif'
            tiff.imsave(low_out,low_tiff_c)
            tiff.imsave(decon_out,decon_tif_c)


def get_data_train(data_dir, data_set, batch_size, pmin, pmax):
    global batch_index, filenames, maxl, indexi

    if len(filenames) == 0: get_filenames(data_dir, data_set)  # 读取数据列表
    maxl = len(filenames)  # 得到file长度

    begin = 0  # 判断每一个batch的范围
    end = batch_size

    x_data = np.array([], np.float32)
    y_data = np.array([], np.float32)  # zero-filled list for 'one hot encoding'
    label_out = []

    a = random.randint(0, 1)

    if "/" in data_dir:
        data_dir=os.sep.join(data_dir.split("/"))

    if "//" in data_dir:
        data_dir=os.sep.join(data_dir.split("//"))

    Input_Path =os.path.join(data_dir,data_set,input_folder,filenames[indexi])
    GroundTruth_Path =os.path.join(data_dir,data_set,ground_truth_folder,filenames[indexi])
    label_out.append(filenames[indexi])

    input_tif1 = tiff.imread(Input_Path)  # 利用tifffile读取tiff文件，[depth,height,width],所以需要考虑是不是需要调整一下顺序
    input_GT1 = tiff.imread(GroundTruth_Path)

    for j in range(begin, end):
        input_tif, input_GT = data_aug_online(input_tif1, input_GT1)

        _,_,normal_input_tif = normalize(input_tif, pmin, pmax)  # last100
        _,_,normal_gt_tif=normalize(input_GT, 0.0, 99.9)
        normal_gt_tif = np.maximum(0, normal_gt_tif)  # ER 1.0,99.5
        [d, h, w] = normal_input_tif.shape
        x_data = np.append(x_data, normal_input_tif)  # 将输入保存到数组中
        y_data = np.append(y_data, normal_gt_tif)  # 将真值保存在数组中

    if indexi + 1 >= maxl:
        indexi = 0
    else:
        indexi = indexi + 1  # update index for the next batch
    x_data_ = x_data.reshape(batch_size, -1)
    y_data_ = y_data.reshape(batch_size, -1)

    return x_data_, y_data_, label_out, [d, h, w]  # 返回数组中的值


def get_data_valid(data_dir, data_set, batch_size, pmin, pmax):
    global batch_index, filenames, maxl, indexi

    if len(filenames) == 0: get_filenames(data_dir, data_set)  # 读取数据列表
    maxl = len(filenames)  # 得到file长度

    begin = 0  # 判断每一个batch的范围
    end = batch_size

    x_data = np.array([], np.float32)
    y_data = np.array([], np.float32)  # zero-filled list for 'one hot encoding'
    label_out = []

    a = random.randint(0, 1)

    if "/" in data_dir:
        data_dir=os.sep.join(data_dir.split("/"))

    if "//" in data_dir:
        data_dir=os.sep.join(data_dir.split("//"))

    Input_Path =os.path.join(data_dir,data_set,input_folder,filenames[indexi])
    GroundTruth_Path =os.path.join(data_dir,data_set,ground_truth_folder,filenames[indexi])
    label_out.append(filenames[indexi])

    input_tif1 = tiff.imread(Input_Path)  # 利用tifffile读取tiff文件，[depth,height,width],所以需要考虑是不是需要调整一下顺序
    input_GT1 = tiff.imread(GroundTruth_Path)

    for j in range(begin, end):
        input_tif, input_GT = input_tif1, input_GT1

        _,_,normal_input_tif = normalize(input_tif, pmin, pmax)  # last100
        _,_,normal_gt_tif=normalize(input_GT, 0.03, 99.7)
        normal_gt_tif = np.maximum(0, normal_gt_tif)  # ER 1.0,99.5
        [d, h, w] = normal_input_tif.shape
        x_data = np.append(x_data, normal_input_tif)  # 将输入保存到数组中
        y_data = np.append(y_data, normal_gt_tif)  # 将真值保存在数组中

    if indexi + 1 >= maxl:
        indexi = 0
    else:
        indexi = indexi + 1  # update index for the next batch
    x_data_ = x_data.reshape(batch_size, -1)
    y_data_ = y_data.reshape(batch_size, -1)

    return x_data_, y_data_, label_out, [d, h, w]  # 返回数组中的值


def get_data_test(data_dir, data_set, batch_size, pmin, pmax):
    global batch_index, filenames

    if len(filenames) == 0: get_filenames(data_dir, data_set)  # 读取数据列表
    max = len(filenames)  # 得到file长度

    begin = batch_index  # 判断每一个batch的范围
    end = batch_index + batch_size

    x_data = np.array([], np.float32)
    label_out = []

    if "/" in data_dir:
        data_dir=os.sep.join(data_dir.split("/"))

    if "//" in data_dir:
        data_dir=os.sep.join(data_dir.split("//"))

    for i in range(begin, end):
        Input_Path = os.path.join(data_dir,data_set,input_folder,filenames[indexi])
        label_out.append(filenames[i])

        input_tif = tiff.imread(Input_Path)  # 利用tifffile读取tiff文件，[depth,height,width],所以需要考虑是不是需要调整一下顺序

        _,max_v, normal_input_tif = normalize(input_tif, pmin, pmax)  # (input_tif-inmin)/(inmax-inmin)
        [d, h, w] = normal_input_tif.shape  # last 100,933

        x_data = np.append(x_data, normal_input_tif)  # 将输入保存到数组中

    if end >= max:
        end = max
        batch_index = 0
    else:
        batch_index += batch_size  # update index for the next batch
    x_data_ = x_data.reshape(batch_size, -1)

    return np.squeeze(max_v), x_data_, label_out, [d, h, w]  # 返回数组中的值



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="RLN-pkg-YueLi", # Replace with your own username
    version="0.1.1",
    author="Yue Li",
    description="RLN package in Incorporating the image formation process into deep learning improves network performance",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MeatyPlus/RLN-package",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


#5#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import faulthandler
faulthandler.enable()
import os
os.environ['TF_CPP_MIN_LOG_LEVEL']='2'
import argparse



def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('--mode',  type=str, default='TS',  help="set the mode")

    return parser, parser.parse_args()


if __name__=="__main__":
    parser, args = parse_args()
    mode=args.mode
    print(mode)

    path_in="C:/Users/tingwang/Desktop/"
    if "/" in path_in:
        path_in=os.sep.join(path_in.split("/"))

    if "//" in path_in:
        path_in=os.sep.join(path_in.split("//"))

    path_out=os.path.join(path_in,'output')
    os.rename(path_out, os.path.join(path_in,'output1'))
